@@ -13,6 +13,14 @@ import time
 import yaml
 
 
+class SlackardFatalError(Exception):
+    pass
+
+
+class SlackardNonFatalError(Exception):
+    pass
+
+
 class Config(object):
     config = {}
 
@@ -69,8 +77,7 @@ class Slackard(object):
         path = self.plugins
         cf = self.config.file
         if path[0] != '/':
-            path = os.path.join(
-                    os.path.dirname(os.path.realpath(cf)), path)
+            path = os.path.join(os.path.dirname(os.path.realpath(cf)), path)
         return path
 
     def _set_import_path(self):
@@ -82,8 +89,15 @@ class Slackard(object):
 
     def _init_connection(self):
         self.slack = slacker.Slacker(self.apikey)
-        r = self.slack.channels.list(self.channel)
-        assert(r.successful)
+        try:
+            r = self.slack.channels.list()
+        except slacker.Error as e:
+            if e.message == 'invalid_auth':
+                raise SlackardFatalError('Invalid API key')
+            raise
+        except Exception as e:
+            raise SlackardNonFatalError(e.message)
+
         c_map = {c['name']: c['id'] for c in r.body['channels']}
         self.chan_id = c_map[self.channel]
 
@@ -119,8 +133,11 @@ class Slackard(object):
                                  self.botnick), re.IGNORECASE)
         h = self.slack.channels.history(self.chan_id, count=1)
         assert(h.successful)
-        ts = h.body['messages'][0]['ts']
         t0 = time.time()
+        if len(h.body['messages']):
+            ts = h.body['messages'][0]['ts']
+        else:
+            ts = t0
 
         while True:
             t1 = time.time()
@@ -129,7 +146,13 @@ class Slackard(object):
                 time.sleep(5.0 - delta_t)
             t0 = time.time()
 
-            messages = self._fetch_messages_since(ts)
+            try:
+                messages = self._fetch_messages_since(ts)
+            except Exception as e:
+                # Possibly an error we can recover from so raise
+                # a non-fatal exception and attempt to recover
+                raise SlackardNonFatalError(e.message)
+
             for message in messages:
                 ts = message['ts']
                 if 'text' in message:
@@ -213,7 +236,6 @@ def usage():
 
 
 def main():
-
     config_file = None
     try:
         config_file = sys.argv[1]
@@ -230,11 +252,26 @@ def main():
 
     try:
         bot = Slackard(config_file)
-        bot.run()
     except Exception as e:
         print('Encountered error: {}'.format(e.message))
         sys.exit(1)
 
+    while True:
+        try:
+            bot.run()
+        except SlackardFatalError as e:
+            print('Fatal error: {}'.format(e.message))
+            sys.exit(1)
+        except SlackardNonFatalError as e:
+            print('Non-fatal error: {}'.format(e.message))
+            delay = 5
+            print('Delaying for {} seconds...'.format(delay))
+            time.sleep(delay)
+            bot._init_connection()
+        #except Exception as e:
+        #    print('Unhandled exception: {}'.format(e.message))
+        #    sys.exit(1)
+
 
 if __name__ == '__main__':
     main()
